using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace ComplexNumbers
{
	// Implements simple drawing operations in user coordinates
	// Use one of the constructors to bind the plotter to a panel or picturebox	

	public class Plotter
	{
		Control outputControl;			// panel or picturebox
		Graphics g;									// graphics object for final drwaing operations
		double xmin, xmax, dx;			// x: min, max and markerstep in user coordinates
		double ymin, ymax, dy;			// y: min, max and markerstep in user coordinates
		int devWidth, devHeight;		// size of drawing area in pixel
		double sx = 1.0;						// x: deviceunits = sx * userunits
		double sy = 1.0;						// y: deviceunits = sy * userunits
		int x0, y0;									// origin (= point 0,0) position in device units
		int x, y;										// current pen position

		const int margin = 25;

		public Plotter(PictureBox pictureBox)
		{
			outputControl = pictureBox;
			g = pictureBox.CreateGraphics();
			Initialize();
		}

		public Plotter(Panel panel)
		{
			outputControl = panel;
			g = panel.CreateGraphics();
			Initialize();
		}

		/*
		 *      o------------------+-----------------------+-> x'
		 *      |    |  y          |                       |
		 *      |    |             |                       |
		 *      |    |             |                       |
		 *      |    |             |                       |
		 *      |    |             |                       |
		 *      |    |             | y0         + x,y      |
		 *      |    |             |                       |
		 *      | x0 |             |                       |
		 *      +----+             |                       |
		 *      |    |             |                       |
		 *      |    |             |                       |
		 *      |    |             |                       |
		 *      |    o-------------+----------------->  x  |
		 *      |                                          |
		 *      +------------------------------------------+
		 *    y'|
		 * 
		 *     x' = x0 + sx * x       sx = device-units / user-units 
		 *     y' = y0 - sy * y
		 * 
		 */

		void Initialize()
		{
			devHeight = outputControl.Height;
			devWidth = outputControl.Width;
			x0 = 20;									// from left
			y0 = devHeight - margin;	// from bottom
			xmin = 0; xmax = devWidth  - 2*margin; dx = 50;
			ymin = 0; ymax = devHeight - 2*margin; dy = 50;
			xmax = dx * Math.Floor(xmax / dx);
			ymax = dy * Math.Floor(ymax / dy);
		}

		int XDev(double xUser)		// x-direction: user to device
		{
			return x0 + (int) (sx * xUser);
		}

		int YDev(double yUser)		// y-direction: user to device
		{
			return y0 - (int) (sy * yUser);
		}

		public void MoveTo(double px, double py)
		{
			x = XDev(px);
			y = YDev(py);
		}

		public void LineTo(double px, double py)
		{
			LineTo(Pens.Black, px, py);
		}

		public void LineTo(Pen pen, double px, double py)
		{
			int xNew;
			int yNew;
			xNew = XDev(px);
			yNew = YDev(py);
			g.DrawLine(pen, x, y, xNew, yNew);
			x = xNew;
			y = yNew;
		}

		public void TextOut(double px, double py, string text)
		{
			int xt = XDev(px);
			int yt = YDev(py);
			g.DrawString(text, outputControl.Font, Brushes.Black, xt, yt);
		}

		public void FillRectangle(Color color, double x0, double y0, double a, double b)
		{
			int left = XDev(x0);
			int top  = YDev(y0 + b);
			int width = (int) (sx*a);
			int height = (int) (sy*b);
			SolidBrush brush = new SolidBrush(color);
			g.FillRectangle(brush, left, top, width, height);
			brush.Dispose();
		}

		public void Mark(Color fgColor, Color bgColor, double x0, double y0, int size)
		{
			int left = XDev(x0) - size/2;
			int top = YDev(y0) - size/2;
			SolidBrush brush = new SolidBrush(bgColor);
			Pen pen = new Pen(fgColor);
			g.FillEllipse(brush, left, top, size, size);
			g.DrawEllipse(pen,   left, top, size, size);
			brush.Dispose();
			pen.Dispose();
		}

		public void DrawXAxis()
		{
			MoveTo(xmin, 0);
			LineTo(xmax, 0);

			string format;
			if (Math.Abs(dx) < 0.1)
				format = "{0:f2}";
			else if (Math.Abs(dx) < 1.0)
				format = "{0:f1}";
			else
				format = "{0:f0}";

			int ixt, iyt;
			string label;
			Size stringSize;
			for (double xt = xmin; xt < 1.05 * xmax; xt += dx)
			{
				if (Math.Abs(xt) < 0.9*dx) continue;
				// draw tic mark
				ixt = XDev(xt);
				iyt = YDev(0);
				g.DrawLine(Pens.Black, ixt, iyt - 2, ixt, iyt + 2);
				// label
				label = string.Format(format, xt);
				stringSize = g.MeasureString(label, outputControl.Font).ToSize();
				g.DrawString(label, outputControl.Font, Brushes.Black, ixt - stringSize.Width / 2, iyt + 4);
			}

		}

		public void DrawYAxis()
		{
			MoveTo(0, ymin);
			LineTo(0, ymax);
			
			string format;
			if (Math.Abs(dy) < 0.1)
				format = "{0:f2}";
			else if (Math.Abs(dy) < 1.0)
				format = "{0:f1}";
			else
				format = "{0:f0}";

			int ixt, iyt;
			string label;
			SizeF labelSize;
			for (double yt = ymin; yt < 1.05 * ymax; yt += dy)
			{
				if (Math.Abs(yt) < 0.9*dy) continue;
				ixt = XDev(0);
				iyt = YDev(yt);
				g.DrawLine(Pens.Black, ixt - 2, iyt, ixt + 2, iyt);
				label = string.Format(format, yt);
				labelSize = g.MeasureString(label, outputControl.Font);
				g.DrawString(label, outputControl.Font, Brushes.Black, 
					ixt - labelSize.Width - 2, iyt - labelSize.Height / 2);
			}

		}

		public void DefineXAxis(double min, double max, double step)
		{
			dx = step;
			xmin = min;
			xmax = max;
			// Calculate the transformation paramaters
			sx = (devWidth - 2*margin) / (xmax - xmin);
			x0 = margin - (int)(sx * xmin);		// in device units
		}

		public void DefineYAxis(double min, double max, double step)
		{
			ymin = min;
			ymax = max;
			dy = step;
			sy = (devHeight - 2*margin) / (ymax - ymin);
			y0 = margin + (int)(sy * ymax);
		}

	}
}
